@@ -2,12 +2,100 @@
 
 import sys
 import json
+import shutil
+from pathlib import Path
 import click
 from typing import Optional
 
 from .config import Config
 from .client import RelayClient
 from .hooks import HooksInstaller
+from .cloudkit_client import (
+    CloudKitClient,
+    CloudKitError,
+    CloudKitNotConfigured,
+    CloudKitUnavailable,
+    DeviceNotFound,
+)
+
+
+def install_scripts() -> dict:
+    """Install hook scripts to ~/.watchcode/ directory.
+
+    Returns dict with 'installed' and 'updated' lists.
+    """
+    from importlib import resources
+
+    result = {"installed": [], "updated": [], "errors": []}
+
+    # Ensure ~/.watchcode exists
+    watchcode_dir = Path.home() / ".watchcode"
+    watchcode_dir.mkdir(exist_ok=True)
+
+    # Scripts to install
+    scripts = [
+        "hook_handler.py",
+        "cloudkit_poller.py",
+        "apns_sender.py",
+        "response_listener.py"
+    ]
+
+    try:
+        # Python 3.9+ way
+        from importlib.resources import files
+        scripts_package = files("watchcode.scripts")
+
+        for script in scripts:
+            try:
+                source = scripts_package.joinpath(script)
+                dest = watchcode_dir / script
+
+                # Read source content
+                content = source.read_text()
+
+                # Check if update needed
+                if dest.exists():
+                    existing = dest.read_text()
+                    if existing == content:
+                        continue  # Already up to date
+                    result["updated"].append(script)
+                else:
+                    result["installed"].append(script)
+
+                # Write script
+                dest.write_text(content)
+                dest.chmod(0o755)  # Make executable
+
+            except Exception as e:
+                result["errors"].append(f"{script}: {e}")
+
+    except ImportError:
+        # Python 3.8 fallback
+        import pkg_resources
+
+        for script in scripts:
+            try:
+                source_path = pkg_resources.resource_filename("watchcode.scripts", script)
+                dest = watchcode_dir / script
+
+                with open(source_path, 'r') as f:
+                    content = f.read()
+
+                if dest.exists():
+                    existing = dest.read_text()
+                    if existing == content:
+                        continue
+                    result["updated"].append(script)
+                else:
+                    result["installed"].append(script)
+
+                dest.write_text(content)
+                dest.chmod(0o755)
+
+            except Exception as e:
+                result["errors"].append(f"{script}: {e}")
+
+    return result
 
 
 @click.group()
@@ -17,26 +105,86 @@ def main():
     pass
 
 
-@main.command()
-def setup():
-    """Interactive setup - configure WatchCode with your setup code."""
-    config = Config()
-    installer = HooksInstaller()
+def _setup_with_cloudkit(config: Config) -> Optional[str]:
+    """Attempt to set up using CloudKit prefix lookup.
 
-    click.echo("WatchCode Setup")
-    click.echo("=" * 50)
+    Returns:
+        The auth token if successful, None if user wants manual entry.
+    """
+    # Check if CloudKit is available and configured
+    if not CloudKitClient.is_available():
+        click.echo("CloudKit client not available (missing dependencies).")
+        click.echo("Falling back to manual setup.")
+        click.echo()
+        return None
+
+    if not CloudKitClient.is_configured():
+        # This is expected for most users - CloudKit bridge is optional
+        return None
+
+    click.echo("CloudKit bridge available - simplified setup enabled!")
+    click.echo()
+    click.echo("Enter the first 4 characters of your setup code.")
+    click.echo("(The full code is shown on your Watch in Settings)")
     click.echo()
 
-    # FIRST: Clean up legacy spam hooks
-    cleanup_result = installer.cleanup_legacy_hooks()
-    if cleanup_result["removed"] or cleanup_result["fixed"]:
-        click.echo("Cleaning up legacy spam hooks...")
-        if cleanup_result["removed"]:
-            click.echo(f"  Removed: {', '.join(cleanup_result['removed'])}")
-        if cleanup_result["fixed"]:
-            click.echo(f"  Fixed: {', '.join(cleanup_result['fixed'])}")
-        click.echo()
+    client = CloudKitClient()
+    max_attempts = 3
 
+    for attempt in range(max_attempts):
+        prefix = click.prompt("Setup code prefix (4 characters)", type=str).strip().upper()
+
+        # Validate prefix format
+        if len(prefix) != 4 or not prefix.isalnum():
+            click.echo("Invalid prefix. Enter exactly 4 alphanumeric characters.")
+            continue
+
+        click.echo(f"Looking up device with prefix '{prefix}'...")
+
+        try:
+            auth_token, device_token = client.find_device_by_prefix(prefix)
+
+            click.echo()
+            click.echo(f"Found device: {config.format_token_display(auth_token)}")
+            click.echo()
+
+            # Confirm with user
+            if click.confirm("Is this your device?", default=True):
+                return auth_token
+            else:
+                click.echo("Try entering the prefix again or use --manual for full code entry.")
+
+        except DeviceNotFound as e:
+            click.echo(f"Not found: {e}")
+            if attempt < max_attempts - 1:
+                click.echo("Try again or use 'watchcode setup --manual' for full code entry.")
+            else:
+                click.echo()
+                click.echo("Could not find device after multiple attempts.")
+                if click.confirm("Switch to manual entry?", default=True):
+                    return None
+                else:
+                    return None  # Exit setup
+
+        except CloudKitUnavailable as e:
+            click.echo(f"CloudKit unavailable: {e}")
+            click.echo("Falling back to manual setup.")
+            return None
+
+        except CloudKitError as e:
+            click.echo(f"CloudKit error: {e}")
+            click.echo("Falling back to manual setup.")
+            return None
+
+    return None
+
+
+def _setup_manual(config: Config) -> str:
+    """Manual setup with full 12-character code entry.
+
+    Returns:
+        The validated auth token.
+    """
     click.echo("To get your setup code:")
     click.echo("1. Open WatchCode on your Apple Watch")
     click.echo("2. Go to Settings")
@@ -53,8 +201,59 @@ def setup():
         else:
             click.echo("Invalid setup code format. Expected: XXXX-XXXX-XXXX (12 alphanumeric characters)")
 
-    # Save auth token (without dashes)
-    clean_token = setup_code.replace("-", "").upper()
+    return setup_code.replace("-", "").upper()
+
+
+@main.command()
+@click.option("--manual", is_flag=True, help="Use manual 12-character code entry instead of CloudKit lookup")
+def setup(manual: bool):
+    """Interactive setup - configure WatchCode with your setup code.
+
+    By default, uses CloudKit to lookup your device with just 4 characters.
+    Use --manual to enter the full 12-character code.
+    """
+    config = Config()
+    installer = HooksInstaller()
+
+    click.echo("WatchCode Setup")
+    click.echo("=" * 50)
+    click.echo()
+
+    # FIRST: Install/update hook scripts to ~/.watchcode/
+    click.echo("Installing hook scripts...")
+    scripts_result = install_scripts()
+    if scripts_result["installed"]:
+        click.echo(f"  Installed: {', '.join(scripts_result['installed'])}")
+    if scripts_result["updated"]:
+        click.echo(f"  Updated: {', '.join(scripts_result['updated'])}")
+    if scripts_result["errors"]:
+        for err in scripts_result["errors"]:
+            click.echo(f"  Error: {err}", err=True)
+    if not scripts_result["installed"] and not scripts_result["updated"]:
+        click.echo("  Scripts already up to date")
+    click.echo()
+
+    # Clean up legacy spam hooks
+    cleanup_result = installer.cleanup_legacy_hooks()
+    if cleanup_result["removed"] or cleanup_result["fixed"]:
+        click.echo("Cleaning up legacy spam hooks...")
+        if cleanup_result["removed"]:
+            click.echo(f"  Removed: {', '.join(cleanup_result['removed'])}")
+        if cleanup_result["fixed"]:
+            click.echo(f"  Fixed: {', '.join(cleanup_result['fixed'])}")
+        click.echo()
+
+    # Attempt CloudKit setup first (unless --manual specified)
+    clean_token = None
+
+    if not manual:
+        clean_token = _setup_with_cloudkit(config)
+
+    # Fall back to manual if CloudKit didn't work
+    if clean_token is None:
+        clean_token = _setup_manual(config)
+
+    # Save auth token
     config.set_auth_token(clean_token)
 
     click.echo()
@@ -206,7 +405,9 @@ def test():
 @main.command()
 @click.option("--event", required=True, help="Event type (e.g., stop, permission_request)")
 @click.option("--requires-action", is_flag=True, help="Notification requires user action")
-def notify(event: str, requires_action: bool):
+@click.option("--message", "message_opt", default=None, help="Custom notification message (overrides HOOK_INPUT)")
+@click.option("--notification-id", "notification_id_opt", default=None, help="Unique notification ID for response correlation")
+def notify(event: str, requires_action: bool, message_opt: Optional[str], notification_id_opt: Optional[str]):
     """Send notification from Claude Code hook (reads hook data from HOOK_INPUT env or stdin)."""
     import os
     config = Config()
@@ -235,8 +436,8 @@ def notify(event: str, requires_action: bool):
             except json.JSONDecodeError:
                 pass  # Ignore invalid JSON
 
-        # Extract data from hook payload
-        message = hook_data.get("message", f"Claude Code: {event}")
+        # Use CLI option if provided, otherwise use hook data
+        message = message_opt if message_opt else hook_data.get("message", f"Claude Code: {event}")
         session_id = hook_data.get("session_id", "unknown")
 
         # Build metadata
@@ -250,8 +451,8 @@ def notify(event: str, requires_action: bool):
             metadata["tool_name"] = hook_data.get("tool_name")
             metadata["tool_input"] = hook_data.get("tool_input")
 
-        # Extract notification_id for response correlation
-        notification_id = hook_data.get("notification_id")
+        # Use CLI option if provided, otherwise use hook data
+        notification_id = notification_id_opt if notification_id_opt else hook_data.get("notification_id")
 
         # Send notification
         client = RelayClient(config)
@@ -351,5 +552,25 @@ def flush():
         sys.exit(1)
 
 
+@main.command("update-scripts")
+def update_scripts():
+    """Update hook scripts in ~/.watchcode/ to latest version."""
+    click.echo("Updating hook scripts...")
+    result = install_scripts()
+
+    if result["installed"]:
+        click.echo(f"Installed: {', '.join(result['installed'])}")
+    if result["updated"]:
+        click.echo(f"Updated: {', '.join(result['updated'])}")
+    if result["errors"]:
+        for err in result["errors"]:
+            click.echo(f"Error: {err}", err=True)
+
+    if not result["installed"] and not result["updated"] and not result["errors"]:
+        click.echo("All scripts already up to date.")
+    else:
+        click.echo("Done!")
+
+
 if __name__ == "__main__":
     main()
